rem + ------------------------------------------------------------------------ +
rem |<                                  BISM                                  >|
rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
rem |                      Binary Insertion Sort (Memory)                      |
rem |                                                                          |
rem | TAOCP 5.2.1. Modified to sort a long word memory index The indexes       |
rem | point to records, also in memory. Individual fields are offset from      |
rem | the main record address.                                                 |
rem |                                                                          |
rem |         ix      is the base of the index                                 |
rem |         n       is the number of items in index (0..N)                   |
rem |         ofs     is an array of offsets pointing to the various fields    |
rem |                    to be included in the sort                            |
rem |         typ%    is an array of corresponding types for those fields      |
rem |                    In the case of strings the typ also specifies the     |
rem |                    comparison type in the msb of the lsw.                |
rem |                                                                          |
rem | Dependencies: McmpMo% and tk COMPM%                                      |
rem + ------------------------------------------------------------------------ +
rem | V0.01, pjw, 2023 Jul 31                                                  |
rem | V0.02, pjw, 2023 Aug 08, changed to nstrg, COMPM% has new parameters     |
rem + ------------------------------------------------------------------------ +
:
DEFine PROCedure BISM(ix, n, ofs, typ%)
LOCal j, i, sl, t
FOR j = 1 TO n - 1
 rem t$ = arr(j%): i% = j% - 1
 t = peek_l(ix + j * 4): i = j - 1
 REPeat sl
  rem IF t$ >= arr(i%): EXIT sl
  if McmpMo%(t, peek_l(ix + i * 4)) >= 0: exit sl
  rem arr(i% + 1) = arr(i%)
  poke_l ix + (i + 1) * 4, peek_l(ix + i * 4)
  rem i% = i% - 1: IF i% <= -1: EXIT sl
  i = i - 1: if i <= -1: exit sl
 END REPeat sl
 rem arr(i% + 1) = t$
 poke_l ix + (i + 1) * 4, t
END FOR j
END DEFine BISM
:
:
rem + ------------------------------------------------------------------------ +
rem |<                                McmpMo%                                 >|
rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
rem |         Compare multiple elements stored in memory using offsets         |
rem |                                                                          |
rem | Subroutine of BISM                                                       |
rem |                                                                          |
rem | Compare each element/field in a record: Comparison ends when the         |
rem | result is either "greater" or "less" or until there are no further       |
rem | elements left to compare in which case the result is "equal".            |
rem |                                                                          |
rem | baseX/Y are abs location of pointers to a record                         |
rem | ofs     is an array containing offsets to fields within record           |
rem | typ%    is an array holding the type of each pair of fields              |
rem |                                                                          |
rem | Dependency: COMPM%                                                       |
rem + ------------------------------------------------------------------------ +
rem | V0.01, pjw, 2023 Jul 30                                                  |
rem | V0.02, pjw, 2023 Aug 08, changed to nstrg, COMPM% has new parameters     |
rem + ------------------------------------------------------------------------ +
:
def fn McmpMo%(baseX, baseY)
loc i%, r%
for i% = 0 to dimn(ofs)
 r% = COMPM%(baseX + ofs(i%), baseY + ofs(i%), typ%(i%))
 if r%: ret r%:    rem Dont return yet if zero..
endfor i%
ret 0:             rem Equals!
enddef McmpMo%
:
:
